package org.gcube.service.idm.keycloack;

import java.rmi.ServerException;
import java.util.List;

import org.gcube.common.security.providers.SecretManagerProvider;
import org.gcube.idm.common.is.InfrastrctureServiceClient;
import org.gcube.idm.common.is.IsServerConfig;
import org.keycloak.admin.client.Keycloak;
import org.keycloak.admin.client.KeycloakBuilder;
import org.keycloak.admin.client.resource.ClientResource;
import org.keycloak.admin.client.resource.RealmResource;
import org.keycloak.representations.idm.ClientRepresentation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import jakarta.ws.rs.NotFoundException;

public class KkClientFactory {
	private static final Logger logger = LoggerFactory.getLogger(KkClientFactory.class);

	private final static String RUNTIME_RESOURCE_NAME = "IAM";
	private final static String CATEGORY = "Service";
	private final static String END_POINT_NAME = "d4science";
	private final static boolean IS_ROOT_SERVICE = true;

	// the singleton obj

	private static KkClientFactory singleton = new KkClientFactory();
	
	public static KkClientFactory getSingleton() {
		if (singleton == null)
			singleton = new KkClientFactory();
		return singleton;
	}

	/**
	 * keycloak configuration obtained from IS in the private constructor
	 * using the singleton pattern, it's retrieved from IS only for the first
	 * access, then kept in the singleton object
	 */
	private IsServerConfig config;

	// allows to configure the factory
	// e.g. from external configuration file
    public void setConfig(IsServerConfig configuration) {
        config = configuration;
    }


	/**
	 * Private constructor
	 * obtains the config from IS
	 */
	private KkClientFactory() {
		logger.info("Building KeycloakAPICredentials object");
		try {
			if (this.config == null) {
				this.config = fetchIsConfig();
			}
			logger.info("KeycloakAPICredentials object built {} - {}", config.getServerUrl(), config.getName());

		} catch (Exception e) {
			logger.error("error obtaining IAM configuration from IS {} ", e);
		}
	}

	// public IsServerConfig configFromINI() throws NotFoundException,
	// ServerException{

	// ApplicationContext appContext = ContextProvider.get();
	// SimpleCredentials credentials =
	// (SimpleCredentials)appContext.authorizationProvider().getCredentials();

	// IsServerConfig cfg = fetchIsConfig();
	// IsServerConfig newConfig = new IsServerConfig(
	// cfg.getServerUrl(),
	// cfg.getName(),
	// credentials.getClientID(), // cfg.getClientId(),
	// credentials.getSecret(), // cfg.getClientSecrxet(),
	// cfg.getProperties()
	// );

	// return newConfig;s

	// }

	public IsServerConfig fetchIsConfig() throws NotFoundException, ServerException {

		IsServerConfig cfg = InfrastrctureServiceClient.serviceConfigFromIS(RUNTIME_RESOURCE_NAME, CATEGORY,
				END_POINT_NAME, IS_ROOT_SERVICE);
		return cfg;

	}

	public static String encodeClientIdContext(String context) {
		return context.replace("/", "%2F");
	}

	public KeycloackApiClient createtKeycloakInstance(String context) {
		return createtKeycloakInstance(this.config, context);
	}

	public static KeycloackApiClient createtKeycloakInstance(IsServerConfig config, String context) {

		Keycloak kclient = KeycloakBuilder.builder()
				.serverUrl(config.getServerUrl())
				.realm(config.getName())
				.grantType(config.getGrantType())
				.clientId(config.getClientId()) //
				.clientSecret(config.getClientSecret()).build();

		return new KeycloackApiClient(kclient, config.getName(), context);
	}

	public RealmResource getKKRealm() {
		String ctx = SecretManagerProvider.get().getContext();
		return getKKRealm(ctx);
	}

	public RealmResource getKKRealm(String ctx) {
		logger.info("Searching client for contex");

		KeycloackApiClient keycloackApiClient = createtKeycloakInstance(ctx);
		RealmResource realm = keycloackApiClient.kclient.realm(keycloackApiClient.realmName);
		return realm;
	}

	public ClientResource getKKClient() {
		String ctx = SecretManagerProvider.get().getContext();
		return getKKClient(ctx);
	}

	public ClientResource getKKClient(String ctx) {
		logger.info("Searching client for contex");

		RealmResource realm = getKKRealm(ctx);

		List<ClientRepresentation> clients = realm.clients().findByClientId(encodeClientIdContext(ctx));

		if (clients.size() == 0) {
			return null;
		}
		String id = clients.get(0).getId();
		return realm.clients().get(id);
	}

	/**
	 * select the ClientResource by name, or current client if clientId parameter is
	 * null;
	 * 
	 * @param clientId
	 * @return
	 */
	public ClientResource getKKClientById(String clientId) {

		if (clientId == null)
			return KkClientFactory.getSingleton().getKKClient();

		RealmResource realmResource = getKKRealm();

		List<ClientRepresentation> clients = realmResource.clients().findByClientId(clientId);

		if (clients.size() == 0) {
			throw new NotFoundException();
		}
		String id = clients.get(0).getId();

		return realmResource.clients().get(id);
	}

	// TODO: REMOVE
	// static IsServerConfig getTestConfig() {
	// String serverUrl = "https://accounts.dev.d4science.org/auth";
	// String realm = "d4science";
	// String clientId = "id.d4science.org";
	// String clientSecret = "";

	// return new IsServerConfig(serverUrl, realm, clientId, clientSecret);
	// }

}
